@@ -86,25 +86,10 @@ pub struct MutationConfig {
 #[derive(Clone, Deserialize, Debug)]
 #[serde(tag = "type")]
 pub enum MutationType {
-    /// A naive branching metaheurstic settings.
-    #[serde(rename(deserialize = "naive-branching"))]
-    NaiveBranching {
-        /// A name of metaheurisic instance.
-        name: String,
-        /// A name of inner metaheurisic instance.
-        inner: String,
-        /// Specifies branching chance settings.
-        chance: BranchingChance,
-        /// Specifies steepness parameter for acceptance probability.
-        steepness: f64,
-        /// Specifies generations range inside a branch.
-        generations: MinMaxConfig,
-    },
-
     /// A metaheuristic which is composition of other metaheuristics with their
     /// probability weights.
-    #[serde(rename(deserialize = "weighted-composite"))]
-    WeightedComposite {
+    #[serde(rename(deserialize = "composite"))]
+    Composite {
         /// A name of metaheurisic instance.
         name: String,
         /// A collection of inner metaheuristics.
@@ -117,16 +102,20 @@ pub enum MutationType {
         /// A name of metaheurisic instance.
         name: String,
         /// Ruin methods.
-        ruins: Vec<ConfigRuinGroup>,
+        ruins: Vec<RuinGroupConfig>,
         /// Recreate methods.
         recreates: Vec<RecreateMethod>,
+        /// Local search config.
+        locals: LocalSearchConfig,
     },
 }
 
 /// A ruin method configuration
 #[derive(Clone, Deserialize, Debug)]
-pub struct ConfigRuinGroup {
+pub struct RuinGroupConfig {
+    /// Ruin methods.
     methods: Vec<RuinMethod>,
+    /// Weight of the group.
     weight: usize,
 }
 
@@ -161,9 +150,9 @@ pub enum RecreateMethod {
     /// Cheapest insertion method.
     #[serde(rename(deserialize = "cheapest"))]
     Cheapest { weight: usize },
-    /// Regret insertion method.
-    #[serde(rename(deserialize = "regret"))]
-    Regret { weight: usize, start: usize, end: usize },
+    /// SkipBest insertion method.
+    #[serde(rename(deserialize = "skip-best"))]
+    SkipBest { weight: usize, start: usize, end: usize },
     #[serde(rename(deserialize = "blinks"))]
     /// Insertion with blinks method.
     Blinks { weight: usize },
@@ -173,6 +162,51 @@ pub enum RecreateMethod {
     /// Nearest neighbour method.
     #[serde(rename(deserialize = "nearest"))]
     Nearest { weight: usize },
+    #[serde(rename(deserialize = "perturbation"))]
+    Perturbation { weight: usize, probability: f64, min: f64, max: f64 },
+    #[serde(rename(deserialize = "regret"))]
+    Regret { weight: usize, start: usize, end: usize },
+}
+
+/// A local search configuration
+#[derive(Clone, Deserialize, Debug)]
+pub struct LocalSearchConfig {
+    /// Pre ruin local search.
+    pre_ruin: LocalSearchGroupConfig,
+    /// Post recreate local search.
+    post_recreate: LocalSearchGroupConfig,
+}
+
+/// A local search group configuration
+#[derive(Clone, Deserialize, Debug)]
+pub struct LocalSearchGroupConfig {
+    /// Probability of the group.
+    probability: f64,
+    /// Amount of times one of operators is applied.
+    times: MinMaxConfig,
+    /// List of operators.
+    operators: Vec<LocalSearchOperator>,
+}
+
+/// A local search configuration.
+#[derive(Clone, Deserialize, Debug)]
+#[serde(tag = "type")]
+pub enum LocalSearchOperator {
+    #[serde(rename(deserialize = "inter-route-best"))]
+    InterRouteBest { weight: usize, noise: NoiseConfig },
+
+    #[serde(rename(deserialize = "inter-route-random"))]
+    InterRouteRandom { weight: usize, noise: NoiseConfig },
+
+    #[serde(rename(deserialize = "intra-route-random"))]
+    IntraRouteRandom { weight: usize, noise: NoiseConfig },
+}
+
+#[derive(Clone, Deserialize, Debug)]
+pub struct NoiseConfig {
+    probability: f64,
+    min: f64,
+    max: f64,
 }
 
 #[derive(Clone, Deserialize, Debug)]
@@ -215,21 +249,6 @@ pub struct MetricsConfig {
     track_population: Option<usize>,
 }
 
-#[derive(Clone, Deserialize, Debug)]
-
-pub struct BranchingConfig {
-    pub chance: BranchingChance,
-    pub steepness: f64,
-    pub generations: MinMaxConfig,
-}
-
-#[derive(Clone, Deserialize, Debug)]
-pub struct BranchingChance {
-    pub normal: f64,
-    pub intensive: f64,
-    pub threshold: f64,
-}
-
 #[derive(Clone, Deserialize, Debug, Eq, PartialEq)]
 pub struct MinMaxConfig {
     pub min: usize,
@@ -309,31 +328,30 @@ fn configure_from_mutation(mut builder: Builder, mutation_config: &Option<Mutati
             NamedMutations::default(),
             |mut mutations, type_cfg| {
                 let (name, mutation): (_, Arc<dyn Mutation + Send + Sync>) = match type_cfg {
-                    MutationType::RuinRecreate { name, ruins, recreates } => {
+                    MutationType::RuinRecreate { name, ruins, recreates, locals } => {
                         let ruin = Box::new(CompositeRuin::new(
                             ruins.iter().map(|g| create_ruin_group(&builder.config.problem, g)).collect(),
                         ));
                         let recreate = Box::new(CompositeRecreate::new(
                             recreates.iter().map(|r| create_recreate_method(r)).collect(),
                         ));
-                        (name.clone(), Arc::new(RuinAndRecreate::new(recreate, ruin)))
+                        (
+                            name.clone(),
+                            Arc::new(RuinAndRecreate::new(
+                                recreate,
+                                ruin,
+                                create_local_search(&locals.pre_ruin),
+                                create_local_search(&locals.post_recreate),
+                            )),
+                        )
                     }
-                    MutationType::WeightedComposite { name, inners } => {
+                    MutationType::Composite { name, inners } => {
                         let inners = inners
                             .iter()
                             .map(|nw| get_mutation_by_name(&mutations, &nw.name).map(|mutation| (mutation, nw.weight)))
                             .collect::<Result<Vec<_>, _>>()?;
-                        (name.clone(), Arc::new(WeightedComposite::new(inners)))
+                        (name.clone(), Arc::new(CompositeMutation::new(inners)))
                     }
-                    MutationType::NaiveBranching { name, inner, chance, steepness, generations } => (
-                        name.clone(),
-                        Arc::new(NaiveBranching::new(
-                            get_mutation_by_name(&mutations, inner)?,
-                            (chance.normal, chance.intensive, chance.threshold),
-                            *steepness,
-                            generations.min..generations.max,
-                        )),
-                    ),
                 };
 
                 mutations.insert(name, mutation);
@@ -366,14 +384,18 @@ fn configure_from_termination(
 fn create_recreate_method(method: &RecreateMethod) -> (Box<dyn Recreate + Send + Sync>, usize) {
     match method {
         RecreateMethod::Cheapest { weight } => (Box::new(RecreateWithCheapest::default()), *weight),
-        RecreateMethod::Regret { weight, start, end } => (Box::new(RecreateWithRegret::new(*start, *end)), *weight),
+        RecreateMethod::SkipBest { weight, start, end } => (Box::new(RecreateWithSkipBest::new(*start, *end)), *weight),
         RecreateMethod::Blinks { weight } => (Box::new(RecreateWithBlinks::<SingleDimLoad>::default()), *weight),
         RecreateMethod::Gaps { weight, min } => (Box::new(RecreateWithGaps::new(*min)), *weight),
         RecreateMethod::Nearest { weight } => (Box::new(RecreateWithNearestNeighbor::default()), *weight),
+        RecreateMethod::Regret { weight, start, end } => (Box::new(RecreateWithRegret::new(*start, *end)), *weight),
+        RecreateMethod::Perturbation { weight, probability, min, max } => {
+            (Box::new(RecreateWithPerturbation::new(*probability, *min, *max)), *weight)
+        }
     }
 }
 
-fn create_ruin_group(problem: &Arc<Problem>, group: &ConfigRuinGroup) -> RuinGroup {
+fn create_ruin_group(problem: &Arc<Problem>, group: &RuinGroupConfig) -> RuinGroup {
     (group.methods.iter().map(|r| create_ruin_method(problem, r)).collect(), group.weight)
 }
 
@@ -401,6 +423,26 @@ fn create_ruin_method(problem: &Arc<Problem>, method: &RuinMethod) -> (Arc<dyn R
     }
 }
 
+fn create_local_search(group: &LocalSearchGroupConfig) -> (Box<dyn LocalSearch + Send + Sync>, f64) {
+    let operators = group
+        .operators
+        .iter()
+        .map::<(Box<dyn LocalSearch + Send + Sync>, usize), _>(|op| match op {
+            LocalSearchOperator::InterRouteBest { weight, noise } => {
+                (Box::new(ExchangeInterRouteBest::new(noise.probability, noise.min, noise.max)), *weight)
+            }
+            LocalSearchOperator::InterRouteRandom { weight, noise } => {
+                (Box::new(ExchangeInterRouteRandom::new(noise.probability, noise.min, noise.max)), *weight)
+            }
+            LocalSearchOperator::IntraRouteRandom { weight, noise } => {
+                (Box::new(ExchangeIntraRouteRandom::new(noise.probability, noise.min, noise.max)), *weight)
+            }
+        })
+        .collect::<Vec<_>>();
+
+    (Box::new(CompositeLocalSearch::new(operators, group.times.min, group.times.max)), group.probability)
+}
+
 fn configure_from_telemetry(builder: Builder, telemetry_config: &Option<TelemetryConfig>) -> Result<Builder, String> {
     const LOG_BEST: usize = 100;
     const LOG_POPULATION: usize = 1000;
